import math


def print_board(boa):
    for row in boa:
        print(*row)


def find_empty(boa):
    n = len(boa)
    for row_idx in range(n):
        for col_idx in range(n):
            if boa[row_idx][col_idx] == 0:
                return row_idx, col_idx
    return None


def valid(boa, row, col, num):
    n = len(boa)
    for i in range(len(boa[row])):
        if col != i and num == boa[row][i]:
            return False

    for i in range(len(boa)):
        if row != i and num == boa[i][col]:
            return False
    step = int(math.sqrt(n))
    start_row = row - row % step
    start_col = col - col % step

    for i in range(start_row,start_row + step):
        for j in range(start_col,start_col + step):
            if boa[i][j] == num and (i,j) != (row,col):
                return False

    return True


def solve_board(boa):
    pos = find_empty(boa)
    if pos is None:
        return True
    row, col = pos
    n = len(boa)
    for num in range(1,n+1):
        if valid(boa,row,col,num):
            boa[row][col] = num

            if solve_board(boa):
                return True

            boa[row][col] = 0

    return False


def am_i_right(boa):
    n = len(boa)
    for row in boa:
        if len(set(row)) != n:
            return False

    for col in range(n):
        set_col = set()
        for row in boa:
            set_col.add(row[col])
        if len(set_col) != n:
            return False

    step = int(math.sqrt(n))
    for start_row in range(0,n-step+1,step):
        for start_col in range(0,n-step+1,step):
            set_box = set()
            for row in range(start_row,start_row+step):
                set_box=set_box.union(set(boa[row][start_col:start_col+step]))
            if len(set_box) != n:
                return False
    return True



import math
import random
from .sudoku_solver import print_board,solve_board,am_i_right


def random_list(n):
    w = [0]*n
    for j in range(n):
        w[j] = j + 1
    for j in range(n-1, -1, -1):
        num = random.randint(0, n-1)
        w[num], w[j] = w[j], w[num]
    return w


def fill_box(boa, row, col):
    n = len(boa)
    step = int(math.sqrt(n))
    start_row = row - row % step
    start_col = col - col % step
    w = random_list(n)
    for i in range(step):
        boa[start_row+i][start_col:start_col+step] = w[i*step:(i+1)*step]


def random_board_generator(n):
    board = []
    for i in range(n):
        z = []
        for j in range(n):
            z.append(0)
        board.append(z)
    step = int(math.sqrt(n))
    for i in range(0,n,step):
        fill_box(board,i,i)
    solve_board(board)
    print_board(board)
    for i in range(n):
        for j in range(n):
            if random.random() < 0.5:
                board[i][j] = 0
    return board


from setuptools import setup
import setuptools

setup(name='sudoku-tool',
      version='0.1',
      description='sudoku toolkit',
      url='https://github.com/Prajeeths18/Basic_Sudoku_Solver.git',
      author='girish-prajeeth',
      author_email='eragon.saphira.100@gmail.com',
      license='MIT',
      packages=setuptools.find_packages()
      )
